# Student2.py ver4.1

print("student.py waking up.")

# Import the Webots simulator API.
from controller import Robot

# Import standard Python libraries.
import math, random, time

# Define the time step in milliseconds between controller updates.
EVENT_LOOP_DT = 10

##############################################################
class Motor(object):
    def __init__(self, motor, joint, init_pos):
        self.motor = motor #this allows control of actual motor
        self.strike_target = 0
        self.retreat_target = 0
        self.strike_speed = 0
        self.strike_index = 'down'
        self.pause1_timer = 0
        self.pause2_timer = 0
        self.pos = init_pos
        self.joint = joint #this allows control of joint

################################################################
class Student(Robot):
    def __init__(self):

        super(Student, self).__init__()
        self.robot_name = "Student"

        # Fetch handles for the sensor.
        self.j0  = self.getPositionSensor('sensor0')
        self.j1  = self.getPositionSensor('sensor1')
        # Specify the sampling rate for the sensor.
        self.j0.enable(EVENT_LOOP_DT)
        self.j1.enable(EVENT_LOOP_DT)
        # Fetch handle for the motor. 
        self.motor0 = self.getMotor('motor0') #movement motor
        self.motor1 = self.getMotor('motor1') #finger motor

        self.state_timer = 0        # timers in milliseconds
        self.state_index = 0        # current state
        self.pos = self.j1.getValue()
        #self.j2.sensor_enable()
        
        self.motor0C = Motor(self.motor0, self.j0, self.j0.getValue()) #motor class
        self.motor1C = Motor(self.motor1, self.j1, self.j1.getValue()) #motor class

        self.motors = {"motor0": self.motor0C, "motor1": self.motor1C} #dict of motor classes
        self.motor_key = "motor1"
        
        # Connect to the radio emitter and receiver.
        self.receiver = self.getReceiver('receiver')
        self.emitter  = self.getEmitter('emitter')
        self.radio_interval = 1
        self.radio_timer = 0
        self.receiver.enable(self.radio_interval)
        
        #communication data
        self.tutor = {"animation":"None", "state":0, "note":"" } #info recieved from the radio
        self.sendAnimation = "None"
        self.sendState = 0
        self.lastNotePlayed = ""
        #send to Tutor: "animationName (int)state"
        #state Key: 0-waiting; 1-performing; 2-finished
        self.time_to_stall = 200
        
        self.calibnotes = {"C": .9, "D": 0.4, "E": 0.1, "F":-0.2, 
                           "G":-0.5, "A":-0.8}
        self.calibUpDown = [0.0, 0.8] #[up, down]
        self.calibNote = "A"
        self.calibmode = False

    #================================================================
    def poll_communication(self):
        self.radio_timer -= EVENT_LOOP_DT
        if self.radio_timer < 0:
            self.radio_timer += self.radio_interval
            while self.receiver.getQueueLength() > 0:
                packet = self.receiver.getData()
                # print("%s Receiver: %s" % (self.robot_name, packet))
                tokens = packet.split()
                if len(tokens) != 3:
                    print("Student recieved malformed packet: %s" % (packet))
                else:
                    #print("S POLLED")
                    name = tokens[0].decode() # convert bytestring to Unicode

                    animation = str(tokens[0])
                    self.tutor["animation"] = animation[2:len(animation)-1]
                    self.tutor["state"] = int(tokens[1])
                    note = str(tokens[2])
                    self.tutor["note"] = note[2:len(note)-1]

                # done with packet processing
                self.receiver.nextPacket()

            # Transmit a status message at the same rate
            animation_token = self.sendAnimation.replace(" ","_")
            status = "%s %d" % (animation_token, self.sendState)

            # emitter requires a bytestring, not a Python Unicode string
            data = status.encode()

            # print("%s emitter: sending %s" % (self.robot_name, data))
            self.emitter.send(data)
    
    def start_keystrike(self, motorkey, strike_position, retreat_position, speed, pause1, pause2):
        self.motor_key = motorkey
        motorC = self.motors[motorkey]
        motorC.strike_target = strike_position
        motorC.retreat_target = retreat_position
        motorC.strike_speed = speed
        motorC.strike_index = 'down'
        motorC.pause1_timer = pause1
        motorC.pause2_timer = pause2

        # This approach uses the low-level PID controller to generate each piecewise linear segment.
        # https://cyberbotics.com/doc/reference/motor?version=R2020b&tab-language=python#field-summary
        motorC.motor.setVelocity(motorC.strike_speed)
        motorC.motor.setPosition(motorC.strike_target)        

        return
    
    def start_keymove(self, key_position, speed, pause2):
        self.start_keystrike("motor0", key_position, key_position, speed, 0, pause2)
    
    def update_keystrike(self):
        motor_key = self.motor_key
        motorC = self.motors[motor_key]
        if motorC.strike_index == 'down':
            if abs(motorC.pos - motorC.strike_target) < 0.05:
                motorC.strike_index = 'pause1'
        
        elif motorC.strike_index == 'pause1':
            motorC.pause1_timer -= EVENT_LOOP_DT
            motorC.motor.setVelocity(0)
            if motorC.pause1_timer <= 0:
                motorC.motor.setPosition(motorC.retreat_target)
                motorC.motor.setVelocity(motorC.strike_speed)
                motorC.strike_index = 'up'
        
        elif motorC.strike_index == 'up':
            if abs(motorC.pos - motorC.retreat_target) < 0.05:
                motorC.strike_index = 'pause2'
        
        elif motorC.strike_index == 'pause2':
            motorC.pause2_timer -= EVENT_LOOP_DT
            motorC.motor.setVelocity(0)
            if motorC.pause2_timer <= 0:
                motorC.motor.setVelocity(motorC.strike_speed)
                motorC.strike_index = 'done'

    def is_strike_done(self):
        """Returns true if an individual gesture is done."""
        motor_key = self.motor_key
        motorC = self.motors[motor_key]
        return motorC.strike_index == 'done'

#===============================================================
    def run(self): 
        #if self.calibmode:
            #self.calib() 
        #self.playnote("C")
        #self.playnote("G")
        while True:
            self.step(EVENT_LOOP_DT)
            self.state_index = 0
            self.poll_communication()
            animation = self.tutor["animation"]
            state = self.tutor["state"]
            note = self.tutor["note"]
            #print(f"Student Recieved: {animation}, {state}, {note} \n")
            #print(f"Student state: {self.sendState} ")
            print(f"Studentsend: {self.sendAnimation}, {self.lastNotePlayed} \n")
            
            if state == 1: #if Tutor performing, Student waits
                self.sendState = 0
                self.poll_communication()
                #print(f"Student state changed to 0: wait\n")
            
            if state == 2: #if Tutor finished, it is Students turn to do an action
                self.sendState = 1
                self.poll_communication()
                print(f"Student state changed to 1: perform\n")
                #self.standardCorrect()
                
                #ACTION TREE:
                if (animation == "standard"):
                    x = random.randint(0,1)
                    if (x == 0):
                        self.stall(200)
                        self.standardCorrect()
                    else:
                        self.standardIncorrect()
                elif animation == "slam":
                    x = random.randint(0,2)
                    if (x == 0):
                        self.laugh()
                    else:
                        self.stall(200)
                        self.standardCorrect()
                elif animation == "multislam":
                    self.stall(200)
                    self.standardCorrect()
                    
                elif animation == "repeat":
                    x = random.randint(0,10)
                    self.repeat()
                    """if (x == 0):
                        self.repeat()
                    else
                        self.standardCorrect()"""
                
                    


######################## ANIMATIONS ############################
    def stall(self, time_to_stall):
        self.state_timer = time_to_stall
        while self.step(EVENT_LOOP_DT) != -1:
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            if timer_expired:
                return -1
################################################################
    def pause(self):
        self.sendAnimation = "standardCorrect"
        self.poll_communication()
        #self.state_index = 0
        while self.step(EVENT_LOOP_DT) != -1:
            # Check the radio and/or network.
            self.poll_communication()
            for motor_key in ["motor1", "motor0"]: 
                #motor_key = self.motor_key
                motorC = self.motors[motor_key]
                # Read sensor value.
                motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()
            
            # Update the activity state machine.    
            #Calibration Tables
            #motor1
            note = self.calibnotes
            #motor0
            up = self.calibUpDown[0]
            down = self.calibUpDown[1]
            
            
            #get correct key:
            correct_key = self.tutor["note"]#string
            self.lastNotePlayed = correct_key
            if correct_key == "":
                print("whoops! there was no note!")
                break
            
            if self.state_index == 0:
                if timer_expired or self.is_strike_done():
                    self.state_timer = 8000
                    self.state_index += 1
                    print("STARTING KEYMOVE")
                    self.start_keymove(0, 0.5, 1000)
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    self.state_timer = 1000
                    self.state_index += 1
                    print("STARTING KEYSTRIKE")
                    self.start_keystrike("motor1", up, up, 1, 100, 1000)
            
            elif self.state_index == 2:
                if timer_expired or self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    print(f"Student state changed to 2: finished\n")
                    self.motor0C.strike_index = "done"
                    self.motor1C.strike_index = "done"
                    self.motor1C.motor.setPosition(up) 
                    self.motor1C.strike_target = up
                    return -1 #animation finished
################################################################
    def calib(self):
        self.sendAnimation = "calib"
        self.poll_communication()
        #self.state_index = 0
        while self.step(EVENT_LOOP_DT) != -1:
            # Check the radio and/or network.
            self.poll_communication()
            for motor_key in ["motor1", "motor0"]: 
                #motor_key = self.motor_key
                motorC = self.motors[motor_key]
                # Read sensor value.
                motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()
            
            # Update the activity state machine.    
            #Calibration Tables
            #motor1
            note = self.calibnotes
            #motor0
            up = self.calibUpDown[0]
            down = self.calibUpDown[1]
            
            if self.state_index == 0:
                if timer_expired or self.is_strike_done():
                    self.state_timer = 8000
                    self.state_index += 1
                    print("STARTING KEYMOVE")
                    self.start_keymove(note[self.calibNote], 0.5, 100)
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    self.state_timer = 1000
                    self.state_index += 1
                    print("STARTING KEYSTRIKE")
                    self.start_keystrike("motor1", down, up, 1, 100, 100)
            
            elif self.state_index == 2:
                if timer_expired or self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    print(f"Student state changed to 2: finished\n")
                    self.motor0C.strike_index = "done"
                    self.motor1C.strike_index = "done"
                    self.motor1C.motor.setPosition(up) 
                    self.motor1C.strike_target = up
                    return -1 #animation finished
###############################################################
    def playnote(self, notestr):
        self.sendAnimation = "playnote"
        self.poll_communication()
        #self.state_index = 0
        while self.step(EVENT_LOOP_DT) != -1:
            self.poll_communication()
            for motor_key in ["motor1", "motor0"]: 
                motorC = self.motors[motor_key]
                motorC.pos = motorC.joint.getValue()
            self.state_timer -= EVENT_LOOP_DT
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            self.update_keystrike()
            note = self.calibnotes
            up = self.calibUpDown[0]
            down = self.calibUpDown[1]
            
            print (self.motor1C.strike_index)
            if self.state_index == 0:
                print("a")
                if self.is_strike_done():
                    self.state_timer = 8000
                    self.state_index += 1
                    self.motor0C.strike_target = note[notestr]
                    self.start_keymove(note[notestr], 0.5, 100)
            
            elif self.state_index == 1:
                print("b")
                if self.is_strike_done():
                    print("DONE MOVE, DOING STRIKE")
                    self.state_timer = 1000
                    self.state_index += 1
                    self.start_keystrike("motor1", down, up, 1, 100, 100)
                    print(self.motor1C.strike_target)
            
            elif self.state_index == 2:
                print("c")
                if self.is_strike_done():
                    print("FINISHED")
                    self.sendState = 2
                    self.poll_communication()
                    print(f"Student state changed to 2: finished\n")
                    self.motor0C.strike_index = "done"
                    self.motor1C.strike_index = "done"
                    self.motor1C.motor.setPosition(up) 
                    self.motor1C.strike_target = up
                    self.state_index = 0
                    return -1 #animation finished

################################################################
    def standardCorrect(self):
        self.sendAnimation = "standardCorrect"
        self.poll_communication()
        #self.state_index = 0
        while self.step(EVENT_LOOP_DT) != -1:
            # Check the radio and/or network.
            self.poll_communication()
            for motor_key in ["motor1", "motor0"]: 
                #motor_key = self.motor_key
                motorC = self.motors[motor_key]
                # Read sensor value.
                motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()
            
            # Update the activity state machine.    
            #Calibration Tables
            #motor1
            note = self.calibnotes
            #motor0
            up = self.calibUpDown[0]
            down = self.calibUpDown[1]
            
            
            #get correct key:
            correct_key = self.tutor["note"]#string
            self.lastNotePlayed = correct_key
            if correct_key == "":
                print("whoops! there was no note!")
                break
            
            if self.state_index == 0:
                if timer_expired or self.is_strike_done():
                    self.state_timer = 8000
                    self.state_index += 1
                    #print("STARTING KEYMOVE")
                    self.start_keymove(note[correct_key], 0.5, 100)
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    self.state_timer = 1000
                    self.state_index += 1
                    #print("STARTING KEYSTRIKE")
                    self.start_keystrike("motor1", down, up, 1, 100, 100)
            
            elif self.state_index == 2:
                if timer_expired or self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    print(f"Student state changed to 2: finished\n")
                    self.motor0C.strike_index = "done"
                    self.motor1C.strike_index = "done"
                    self.motor1C.motor.setPosition(up) 
                    self.motor1C.strike_target = up
                    return -1 #animation finished

################################################################
    def repeat(self):
        self.sendAnimation = "repeat"
        self.poll_communication()
        self.state_index = 0
        while self.step(EVENT_LOOP_DT) != -1:
            motor_key = self.motor_key
            motorC = self.motors[motor_key]
            # Read sensor value.
            motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()
            
            # Update the activity state machine.    
            #Calibration Tables
            #motor1
            note = self.calibnotes
            #motor0
            up = self.calibUpDown[0]
            down = self.calibUpDown[1]
            
            if self.state_index == 0:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    #print("STARTING KEYMOVE")
                    self.start_keymove(note[self.lastNotePlayed], 0.5, 100)
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    #print("STARTING KEYSTRIKE")
                    self.start_keystrike("motor1", down, up, 2, 100, 100)
            
            elif self.state_index == 2:
                if timer_expired or self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    print(f"Student state changed to 2: finished\n")
                    self.motor0C.strike_index = "done"
                    self.motor1C.strike_index = "done"
                    self.motor1C.motor.setPosition(up) 
                    self.motor1C.strike_target = up
                    return -1 #animation finished

################################################################
    def standardIncorrect(self):
        self.sendAnimation = "standardIncorrect"
        self.poll_communication()
        self.state_index = 0
        while self.step(EVENT_LOOP_DT) != -1:
            motor_key = self.motor_key
            motorC = self.motors[motor_key]
            motorC.pos = motorC.joint.getValue()
            self.state_timer -= EVENT_LOOP_DT
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            self.update_keystrike()
            
            # Update the activity state machine.    
            #Calibration Tables
            #motor1
            note = self.calibnotes
            #motor0
            up = self.calibUpDown[0]
            down = self.calibUpDown[1]
            
            #get correct key:
            correct_key = self.tutor["note"]#string
            if correct_key == "":
                print("whoops! there was no note!")
                break
            
            if self.state_index == 0:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    #choose some key that is not the correct_key
                    randnote = random.choice(list(note.keys()))
                    while (randnote == correct_key):
                        randnote = random.choice(list(note.keys()))
                    print(f"PLaying incorrect randnote: {randnote}")
                    self.lastNotePlayed = randnote
                    self.start_keymove(note[randnote], 3, 0)
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike("motor1", down, up, 1, 100, 100)
            
            elif self.state_index == 2:
                if timer_expired or self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    self.motor0C.strike_index = "done"
                    self.motor1C.strike_index = "done"
                    self.motor1C.motor.setPosition(up) 
                    self.motor1C.strike_target = up
                    return -1 #animation finished
###############################################################
    def laugh(self):
        self.sendAnimation = "laugh"
        self.poll_communication()
        self.state_index = 0
        while self.step(EVENT_LOOP_DT) != -1:
            motor_key = self.motor_key
            motorC = self.motors[motor_key]
            # Read sensor value.
            motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()
            
            # Update the activity state machine.    
            #Calibration Tables
            #motor1
            note = self.calibnotes
            #motor0
            up = self.calibUpDown[0]
            down = self.calibUpDown[1]         
            
            if self.state_index == 0:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 200
                    self.state_index += 1
                    self.start_keystrike("motor1", -down/4, up, 1, 0, 0)
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 200
                    self.state_index += 1
                    self.start_keystrike("motor1", -down/4, up, 1, 0, 0)
            
            elif self.state_index == 2:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 200
                    self.state_index += 1
                    self.start_keystrike("motor1", up, up, 1, 50, 100)
            
            elif self.state_index == 3:
                if timer_expired or self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    return -1 #animation finished
#############################################################
    def taunt(self):
        self.sendAnimation = "taunt"
        self.poll_communication()
        self.state_index = 0
        while self.step(EVENT_LOOP_DT) != -1:
            motor_key = self.motor_key
            motorC = self.motors[motor_key]
            # Read sensor value.
            motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()
            
            # Update the activity state machine.    
            #Calibration Tables
            #motor1
            note = self.calibnotes
            #motor0
            up = self.calibUpDown[0]
            down = self.calibUpDown[1]        
            
            if self.state_index == 0:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 200
                    self.state_index += 1
                    self.start_keymove(note["C"], 3, 0)
                    self.start_keystrike("motor1", down/2, down/2, .5, 300, 300)
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 200
                    self.state_index += 1
                    self.start_keymove(note["E"], 3, 0)
                    self.start_keystrike("motor1", up, up, .5, 300, 300)
            
            elif self.state_index == 2:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 200
                    self.state_index += 1
                    self.start_keymove(note["C"], 3, 0)
                    self.start_keystrike("motor1", down/2, down/2, .5, 300, 300)
            
            elif self.state_index == 3:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 200
                    self.state_index += 1
                    self.start_keymove(note["E"], 3, 0)
                    self.start_keystrike("motor1", up, up, .5, 0, 300)
            
            elif self.state_index == 4:
                if timer_expired or self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    print(f"Student state changed to 2: finished\n")
                    self.motor0C.strike_index = "done"
                    self.motor1C.strike_index = "done"
                    self.motor1C.motor.setPosition(up) 
                    self.motor1C.strike_target = up
                    return -1 #animation finished
    
                    

# Start the script.
robot = Student()
robot.run()



