# Student2.py ver3 (ROTATIONAL MOTORS)

print("student.py waking up.")

# Import the Webots simulator API.
from controller import Robot

# Import standard Python libraries.
import math, random, time

# Define the time step in milliseconds between controller updates.
EVENT_LOOP_DT = 10

################################################################
class Student(Robot):
    def __init__(self):

        super(Student, self).__init__()

        # Fetch handles for the sensor.
        self.j0  = self.getPositionSensor('sensor0')
        self.j1  = self.getPositionSensor('sensor1')
        # Specify the sampling rate for the sensor.
        self.j0.enable(EVENT_LOOP_DT)
        self.j1.enable(EVENT_LOOP_DT)
        # Fetch handle for the motor. 
        self.motor0 = self.getMotor('motor0')
        self.motor1 = self.getMotor('motor1')

        self.state_timer = 0        # timers in milliseconds
        self.state_index = 0        # current state
        self.pos = self.j1.getValue()
        #self.j2.sensor_enable()
        
        self.strike_target = 0
        self.retreat_target = 0
        self.strike_speed = 0
        self.strike_index = 'down'
        self.pause1_timer = 0
        self.pause2_timer = 0
        self.motors = [self.motor0, self.motor1]
        self.motorIndex = 0
        

    #================================================================
    def start_keystrike(self, motorIndex, strike_position, retreat_position, speed, pause1, pause2):
        self.motorIndex = motorIndex
        self.strike_target = strike_position
        self.retreat_target = retreat_position
        self.strike_speed = speed
        self.strike_index = 'down'
        self.pause1_timer = pause1
        self.pause2_timer = pause2

        # This approach uses the low-level PID controller to generate each piecewise linear segment.
        # https://cyberbotics.com/doc/reference/motor?version=R2020b&tab-language=python#field-summary
        self.motors[motorIndex].setVelocity(self.strike_speed)
        self.motors[motorIndex].setPosition(self.strike_target)        

        return
    
    
    def update_keystrike(self):
        motorIndex = self.motorIndex
        if self.strike_index == 'down':
            if abs(self.pos[motorIndex] - self.strike_target) < 0.05:
                self.strike_index = 'pause1'
        
        elif self.strike_index == 'pause1':
            self.pause1_timer -= EVENT_LOOP_DT
            self.motors[motorIndex].setVelocity(0)
            if self.pause1_timer <= 0:
                self.motors[motorIndex].setPosition(self.retreat_target)
                self.motors[motorIndex].setVelocity(self.strike_speed)
                self.strike_index = 'up'
        
        elif self.strike_index == 'up':
            if abs(self.pos[motorIndex] - self.retreat_target) < 0.05:
                self.strike_index = 'pause2'
        
        elif self.strike_index == 'pause2':
            self.pause2_timer -= EVENT_LOOP_DT
            self.motors[motorIndex].setVelocity(0)
            if self.pause2_timer <= 0:
                self.motors[motorIndex].setVelocity(self.strike_speed)
                self.strike_index = 'done'

    def is_strike_done(self):
        """Returns true if an individual gesture is done."""
        return self.strike_index == 'done'
#===============================================================
    def run(self):

        while self.step(EVENT_LOOP_DT) != -1:
        
        # self.sim_time = self.getTime()
            # Read sensor value.
            self.pos = [self.j0.getValue(), self.j1.getValue()]
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT

            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0

            # Update the motor controls
            self.update_keystrike()
            
            # Update the activity state machine.    
                   
            if self.state_index == 0:
                # if the timing is all derived from the low-level primitive, you may not need a timer here, but
                # it still might be a useful animation control
                if timer_expired or self.is_strike_done():
                    self.state_timer += 2000
                    self.state_index += 1
                    self.start_keystrike(1, 0, 0, 0, 0, 2000)
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(0, .2, .2, .5, 0, 300)
            
            elif self.state_index == 2:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(1, 0.8, 0.0, 1.2, 200, 1200)
            
            elif self.state_index == 3:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(0, -.2, -.2, 1, 100, 100)
            
            elif self.state_index == 4:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(0, .1, .1, 1, 100, 100)
            
            elif self.state_index == 5:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 200
                    self.state_index += 1
                    self.start_keystrike(0, 0.0, 0, 1, 100, 0)
            
            elif self.state_index == 6:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(1, 0.8, 0.0, 2.4, 100, 1500)
            
            elif self.state_index == 7:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(0, -0.1, -0.1, 1, 100, 200)
            
            elif self.state_index == 8:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(0, 0, 0, 1, 100, 0)
            
            elif self.state_index == 9:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(1, 0.8, 0.0, 1.4, 100, 1500)
            
            elif self.state_index == 10:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(1, 0.8, 0.0, 1.4, 100, 1000)
            
            elif self.state_index == 11:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(0, -.2, -.2, .05, 0, 200)
            
            elif self.state_index == 12:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(1, .4, 0.4, .1, 0, 0)
            
            elif self.state_index == 13:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(0, 0.0, 0.0, 2, 0, 0)
            
            elif self.state_index == 14:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(1, 0.8, 0.0, 2.4, 100, 1500)
                    
            # ....
            print(f"S: {self.state_index}, {self.strike_speed}, {self.strike_index}, {self.strike_target}, {self.retreat_target}")

################################################################
# Start the script.
robot = Student()
robot.run()


