# Tutor2.py ver1

print("Tutor.py waking up.")

# Import the Webots simulator API.
from controller import Robot

# Import standard Python libraries.
import math, random, time

# Define the time step in milliseconds between controller updates.
EVENT_LOOP_DT = 10

################################################################
class Tutor(Robot):
    def __init__(self):

        super(Tutor, self).__init__()

        # Fetch handles for the sensor.
        self.j1  = self.getPositionSensor('sensor1')
        # Specify the sampling rate for the sensor.
        self.j1.enable(EVENT_LOOP_DT)
        # Fetch handle for the motor. 
        self.motor1 = self.getMotor('motor1')

        self.state_timer = 0        # timers in milliseconds
        self.state_index = -1        # current state
        self.pos = self.j1.getValue()
        return

    #================================================================
    #moves to 'pos'
    def goTo(self, pos, speed=.1):
        if (abs(pos - self.pos) > 0.03):
            diff = pos - self.pos
            if diff<0:
                self.motor1.setPosition(self.pos - speed)
            else:
                self.motor1.setPosition(self.pos + speed)
        else:
            self.motor1.setPosition(self.pos)
        return

    #moves to rest position (maxStop = 0.4)
    def returns(self, speed=.1):
        self.motor1.setPosition(self.pos + speed)
        return

    #================================================================
    def run(self):
        # If the controller returns -1, the simulator is quitting.
        while self.step(EVENT_LOOP_DT) != -1:
            # Read simulator clock time.
            self.sim_time = self.getTime()
            # Read sensor value.
            self.pos = self.j1.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            
            print (self.state_index)
            timer_expired = False
            #initialize different time lengths for different actions
            if self.state_timer < 0:
                if self.state_index in [2, 8]:
                    self.state_timer += 100
                elif self.state_index in [1,3,9]:
                    self.state_timer += 1200
                elif self.state_index in [4,5,6,7]:
                    self.state_timer += 300
                else:
                    self.state_timer += 800
                timer_expired = True
                
            if self.state_index == -1:
                print("Init state, entering cycle.")
                self.state_index = 0
                
            elif self.state_index == 0:
                if timer_expired:
                    self.state_index += 1
            
            #First reaction: Calm and efficient
            elif self.state_index == 1:
                self.goTo(0, 0.05)
                if timer_expired:
                    self.state_index += 1

            elif self.state_index == 2:
                self.returns(0.05)
                if timer_expired:
                    self.state_index += 1
            
            #Second reaction: quick and angry
            elif self.state_index == 3:
                self.goTo(0, .20)
                if timer_expired:
                    self.state_index += 1

            elif self.state_index == 4:
                self.returns(0.3)
                if timer_expired:
                    self.state_index += 1
                    
            #Third reaction: jerky and hesitant
            elif self.state_index == 5:
                self.goTo(.2, .07)
                if timer_expired:
                    self.state_index += 1

            elif self.state_index == 6:
                self.goTo(.26, .1)
                if timer_expired:
                    self.state_index += 1
            
            elif self.state_index == 7:
                self.goTo(.17, .02)
                if timer_expired:
                    self.state_index += 1
            
            elif self.state_index == 8:
                self.goTo(.1, .1)
                if timer_expired:
                    self.state_index += 1
            
            elif self.state_index == 9:
                self.goTo(0, .23)
                if timer_expired:
                    self.state_index += 1
            
            elif self.state_index == 10:
                self.returns(.23)
                if timer_expired:
                    self.state_index = 1

            else:
                print("%s: invalid state, resetting." % (self.robot_name))
                self.state_index = 0
                
        return


################################################################
# Start the script.
robot = Tutor()
robot.run()
