# Tutor2.py ver4.1

print("Tutor.py waking up.")

# Import the Webots simulator API.
from controller import Robot

# Import standard Python libraries.
import math, random, time

# Define the time step in milliseconds between controller updates.
EVENT_LOOP_DT = 10

###############################################################
class Motor(object):
    def __init__(self, motor, joint, init_pos):
        self.motor = motor #this allows control of actual motor
        self.strike_target = 0
        self.retreat_target = 0
        self.strike_speed = 0
        self.strike_index = 'down'
        self.pause1_timer = 0
        self.pause2_timer = 0
        self.pos = init_pos
        self.joint = joint #this allows control of joint
        

################################################################
class Tutor(Robot):
    def __init__(self):

        super(Tutor, self).__init__()
        self.robot_name = "Tutor"
        
        #Calibration variables (must be adjusted for physical model)
        self.key_space = 0
        self.motor_movement = 0
         
        
        # Fetch handles for the sensor.
        self.j0  = self.getPositionSensor('sensor0')
        self.j1  = self.getPositionSensor('sensor1')
        self.j2  = self.getPositionSensor('sensor2')
        self.j3  = self.getPositionSensor('sensor3')
        # Specify the sampling rate for the sensor.
        self.j0.enable(EVENT_LOOP_DT)
        self.j1.enable(EVENT_LOOP_DT)
        self.j2.enable(EVENT_LOOP_DT)
        self.j3.enable(EVENT_LOOP_DT)
        # Fetch handle for the motor. 
        self.motor0 = self.getMotor('motor0') #movement motor
        self.motor1 = self.getMotor('motor1') #finger motors
        self.motor2 = self.getMotor('motor2')
        self.motor3 = self.getMotor('motor3')

        self.state_timer = 0        # timers in milliseconds
        self.state_index = 0        # current state
        self.pos = self.j1.getValue()
        #self.j2.sensor_enable()
        
        self.motor0C = Motor(self.motor0, self.j0, self.j0.getValue()) #motor class
        self.motor1C = Motor(self.motor1, self.j1, self.j1.getValue()) #motor class
        self.motor2C = Motor(self.motor2, self.j2, self.j2.getValue())
        self.motor3C = Motor(self.motor3, self.j3, self.j3.getValue())

        self.motors = {"motor0": self.motor0C, "motor1": self.motor1C, "motor2": self.motor2C, "motor3": self.motor3C} #dict of motor classes
        self.motor_key = "motor1"
        
        # Connect to the radio emitter and receiver.
        self.receiver = self.getReceiver('receiver')
        self.emitter  = self.getEmitter('emitter')
        self.radio_interval = 1
        self.radio_timer = 0
        self.receiver.enable(self.radio_interval)
        
        #communication data
        self.student = {"animation":"None", "state":0} #info recieved from the radio
        self.sendAnimation = "None"
        self.sendState = 0
        self.sendNote = ""
        #send to student: "animationName (int)state noteName"
        #state Key: 0-waiting; 1-performing; 2-finished
        
        self.calibnotes = {"C": 0.9, "D":.40, 
                           "E":00, "F":-0.36, "G":-.7, "A":-.98 }
        self.calibUpDown = [0.0, 0.8] #[up, down]
        self.calibNote = "A"
        self.calibMode = False
        self.time_to_stall = 200
        

    #================================================================
    def poll_communication(self):
        self.radio_timer -= EVENT_LOOP_DT
        if self.radio_timer < 0:
            self.radio_timer += self.radio_interval
            while self.receiver.getQueueLength() > 0:
                packet = self.receiver.getData()
                # print("%s Receiver: %s" % (self.robot_name, packet))
                tokens = packet.split()
                if len(tokens) != 2:
                    print("Tutor revieved malformed packet: %s" % (packet))
                else:
                    name = tokens[0].decode() # convert bytestring to Unicode
                    
                    animation = str(tokens[0])
                    self.student["animation"] = animation[2:len(animation)-1]
                    self.student["state"] = int(tokens[1])

                # done with packet processing
                self.receiver.nextPacket()

            # Transmit a status message at the same rate
            animation_token = self.sendAnimation.replace(" ","_")
            status = "%s %d %s" % (animation_token, self.sendState, self.sendNote)

            # emitter requires a bytestring, not a Python Unicode string
            data = status.encode()

            # print("%s emitter: sending %s" % (self.robot_name, data))
            self.emitter.send(data)
    
    
    def start_keystrike(self, motorkey, strike_position, retreat_position, speed, pause1, pause2):
        self.motor_key = motorkey
        motorC = self.motors[motorkey]
        motorC.strike_target = strike_position
        motorC.retreat_target = retreat_position
        motorC.strike_speed = speed
        motorC.strike_index = 'down'
        motorC.pause1_timer = pause1
        motorC.pause2_timer = pause2

        # This approach uses the low-level PID controller to generate each piecewise linear segment.
        # https://cyberbotics.com/doc/reference/motor?version=R2020b&tab-language=python#field-summary
        motorC.motor.setVelocity(motorC.strike_speed)
        motorC.motor.setPosition(motorC.strike_target)        

        return
        
    def start_keymove(self, key_position, speed, pause2):
        self.start_keystrike("motor0", key_position, key_position, speed, 0, pause2)
    
    
    def update_keystrike(self):
        for motor_key in ["motor1", "motor2", "motor3", "motor0"]:
            #motor_key = self.motor_key
            motorC = self.motors[motor_key]
            if motorC.strike_index == 'down':
                #print(f"{motorC.pos} - {motorC.strike_target}")
                if abs(motorC.pos - motorC.strike_target) < 0.05:
                    motorC.strike_index = 'pause1'
            
            elif motorC.strike_index == 'pause1':
                motorC.pause1_timer -= EVENT_LOOP_DT
                motorC.motor.setVelocity(0)
                if motorC.pause1_timer <= 0:
                    motorC.motor.setPosition(motorC.retreat_target)
                    motorC.motor.setVelocity(motorC.strike_speed)
                    motorC.strike_index = 'up'
            
            elif motorC.strike_index == 'up':
                if abs(motorC.pos - motorC.retreat_target) < 0.05:
                    motorC.strike_index = 'pause2'
            
            elif motorC.strike_index == 'pause2':
                motorC.pause2_timer -= EVENT_LOOP_DT
                #motorC.motor.setVelocity(0)
                if motorC.pause2_timer <= 0:
                    motorC.motor.setVelocity(motorC.strike_speed)
                    motorC.strike_index = 'done'

    def is_strike_done(self):
        """Returns true if an individual gesture is done."""
        """for motor_key in ["motor1", "motor2", "motor3", "motor0"]:
            #motor_key = self.motor_key
            motorC = self.motors[motor_key]"""
        
        return ((self.motors["motor1"].strike_index == 'done') and 
        (self.motors["motor2"].strike_index == 'done') and 
        (self.motors["motor3"].strike_index == 'done') and 
        (self.motors["motor0"].strike_index == 'done'))
    #================================================================
    def run(self):
        #if self.calibmode:
            #self.calib()
        self.sendState = 1
        self.poll_communication()
        self.standard()
        while True:
            self.step(EVENT_LOOP_DT)
            self.state_index = 0
            self.poll_communication()
            animation = self.student["animation"]
            state = self.student["state"]
            #print(f"Tutor Recieved: {animation}, {state}\n")
            #print(f"Tutor state: {self.sendState} ")
            print(f"Tutorsend: {self.sendAnimation}, {self.sendNote}, {self.sendState} \n")
            print(f"Tutorrec: {animation}, {state}\n")
            
            
            if state == 1: #if Student performing, change state to wait
                self.sendState = 0
                self.poll_communication()
                #print(f"Tutor state changed to 0: wait\n")
            
            if state == 2: #if Student finshed, it is Tutors turn to do an action
                self.sendState = 1
                self.poll_communication()
                print(f"Tutor state changed to 1: perform\n")
                #self.standard()
                
                #ACTION TREE:
                if (animation == "standardCorrect"):
                    self.stall(200)
                    self.standard()
                elif animation == "standardIncorrect":
                    x = random.randint(0,10)
                    if (x < 8):
                        self.repeat()
                    else:
                        self.slam()
                elif (animation == "laugh" or animation == "taunt"):
                    x = random.randint(0,10)
                    if (x == 0):
                        self.slam()
                    elif (x == 1):
                        self.multislam()
                    elif (x == 2):
                        self.standard()
                    else:
                        self.repeat()
                elif animation == "repeat":
                    x = random.randint(0,10)
                    if (x < 3):
                        self.slam()
                    else:
                        self.repeat()
                        
##################################################################
    def stall(self, time_to_stall):
        self.state_timer = time_to_stall
        while self.step(EVENT_LOOP_DT) != -1:
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            if timer_expired:
                return -1
    
    def pause(self):
        self.sendAnimation = "standard"
        self.poll_communication()
        while self.step(EVENT_LOOP_DT) != -1:
            # Check the radio and/or network.
            self.poll_communication()
            for motor_key in ["motor1", "motor2", "motor3", "motor0"]: 
                #motor_key = self.motor_key
                motorC = self.motors[motor_key]
                # Read sensor value.
                motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()
            #Calibration Tables
            #motor3 placement (left finger from tutor POV)
            note = self.calibnotes
            #motor0
            up = self.calibUpDown[0]
            down = self.calibUpDown[1] 
            # Update the activity state machine.    

            if self.state_index == 0:
                # if the timing is all derived from the low-level primitive, you may not need a timer here, but
                # it still might be a useful animation control
                if timer_expired or self.is_strike_done():
                    self.state_timer = 6000
                    self.state_index += 1
                    #choose random key
                    randnote = random.choice(list(note.keys()))
                    self.sendNote = randnote
                    self.poll_communication()
                    self.start_keymove(0, 0.5, 1000)
                    #print(f"moving to {randnote}")
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    if timer_expired: 
                        print("timer expired!")
                    #print("move is done. Doing strike")
                    self.state_timer = 2000
                    self.state_index += 1
                    self.start_keystrike("motor1", up, up, 1, 100, 1000)
                    #print(self.motor1C.strike_index)
                        
            elif self.state_index == 2:
                if self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    print(f"Tutor state changed to 2: finished\n")
                    self.poll_communication()
                    self.motor0C.strike_index = "done"
                    self.motor1C.strike_index = "done"
                    self.motor2C.strike_index = "done"
                    self.motor3C.strike_index = "done"
                    self.motor1C.motor.setPosition(up) 
                    self.motor2C.motor.setPosition(up) 
                    self.motor3C.motor.setPosition(up) 
                    self.motor1C.strike_target = up
                    self.motor2C.strike_target = up
                    self.motor3C.strike_target = up
                    
                    return -1 #animation finished
            #print(f"motor0: {self.motor0C.strike_index}")
            #print(f"motor1: {self.motor1C.strike_index}")
################################################################## 
    def calib(self):
        self.sendAnimation = "calib"
        self.poll_communication()
        while self.step(EVENT_LOOP_DT) != -1:
            # Check the radio and/or network.
            self.poll_communication()
            for motor_key in ["motor1", "motor2", "motor3", "motor0"]: 
                #motor_key = self.motor_key
                motorC = self.motors[motor_key]
                # Read sensor value.
                motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()
            #Calibration Tables
            #motor3 placement (left finger from tutor POV)
            note = self.calibnotes
            #motor0
            up = self.calibUpDown[0]
            down = self.calibUpDown[1] 
            # Update the activity state machine.    

            if self.state_index == 0:
                # if the timing is all derived from the low-level primitive, you may not need a timer here, but
                # it still might be a useful animation control
                if timer_expired or self.is_strike_done():
                    self.state_timer = 6000
                    self.state_index += 1
                    #choose random key
                    randnote = random.choice(list(note.keys()))
                    self.sendNote = randnote
                    self.poll_communication()
                    self.start_keymove(note[self.calibNote], 0.5, 0)
                    #print(f"moving to {randnote}")
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    if timer_expired: 
                        print("timer expired!")
                    #print("move is done. Doing strike")
                    self.state_timer = 2000
                    self.state_index += 1
                    self.start_keystrike("motor1", down, up, 1, 100, 0)
                    #print(self.motor1C.strike_index)
                        
            elif self.state_index == 2:
                if self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    print(f"Tutor state changed to 2: finished\n")
                    self.poll_communication()
                    self.motor0C.strike_index = "done"
                    self.motor1C.strike_index = "done"
                    self.motor2C.strike_index = "done"
                    self.motor3C.strike_index = "done"
                    self.motor1C.motor.setPosition(up) 
                    self.motor2C.motor.setPosition(up) 
                    self.motor3C.motor.setPosition(up) 
                    self.motor1C.strike_target = up
                    self.motor2C.strike_target = up
                    self.motor3C.strike_target = up   
###################################################################    
    def standard(self):
        self.sendAnimation = "standard"
        self.poll_communication()
        while self.step(EVENT_LOOP_DT) != -1:
            # Check the radio and/or network.
            self.poll_communication()
            for motor_key in ["motor1", "motor2", "motor3", "motor0"]: 
                #motor_key = self.motor_key
                motorC = self.motors[motor_key]
                # Read sensor value.
                motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()
            #Calibration Tables
            #motor3 placement (left finger from tutor POV)
            note = self.calibnotes
            #motor0
            up = self.calibUpDown[0]
            down = self.calibUpDown[1] 
            # Update the activity state machine.    

            if self.state_index == 0:
                # if the timing is all derived from the low-level primitive, you may not need a timer here, but
                # it still might be a useful animation control
                if timer_expired or self.is_strike_done():
                    self.state_timer = 6000
                    self.state_index += 1
                    #choose random key
                    randnote = random.choice(list(note.keys()))
                    self.sendNote = randnote
                    self.poll_communication()
                    self.start_keymove(note[randnote], 0.5, 0)
                    #print(f"moving to {randnote}")
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    if timer_expired: 
                        print("timer expired!")
                    #print("move is done. Doing strike")
                    self.state_timer = 2000
                    self.state_index += 1
                    self.start_keystrike("motor1", down, up, 1, 100, 0)
                    #print(self.motor1C.strike_index)
                        
            elif self.state_index == 2:
                if self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    print(f"Tutor state changed to 2: finished\n")
                    self.poll_communication()
                    self.motor0C.strike_index = "done"
                    self.motor1C.strike_index = "done"
                    self.motor2C.strike_index = "done"
                    self.motor3C.strike_index = "done"
                    self.motor1C.motor.setPosition(up) 
                    self.motor2C.motor.setPosition(up) 
                    self.motor3C.motor.setPosition(up) 
                    self.motor1C.strike_target = up
                    self.motor2C.strike_target = up
                    self.motor3C.strike_target = up
                    
                    return -1 #animation finished
            #print(f"motor0: {self.motor0C.strike_index}")
            #print(f"motor1: {self.motor1C.strike_index}")

##################################################################        
    def repeat(self):
        self.sendAnimation = "repeat"
        self.poll_communication()
        while self.step(EVENT_LOOP_DT) != -1:
            # Check the radio and/or network.
            self.poll_communication()
            for motor_key in ["motor1", "motor2", "motor3", "motor0"]: 
                #motor_key = self.motor_key
                motorC = self.motors[motor_key]
                # Read sensor value.
                motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()
            #Calibration Tables
            #motor3 placement (left finger from tutor POV)
            note = self.calibnotes
            #motor0
            up = self.calibUpDown[0]
            down = self.calibUpDown[1] 
            # Update the activity state machine.    

            if self.state_index == 0:
                # if the timing is all derived from the low-level primitive, you may not need a timer here, but
                # it still might be a useful animation control
                if timer_expired or self.is_strike_done():
                    self.state_timer = 6000
                    self.state_index += 1
                    self.start_keymove(note[self.sendNote], 0.5, 0) #uses whatever key was last played (sendNote)
                    #print(f"moving to {randnote}")
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    if timer_expired: 
                        print("timer expired!")
                    #print("move is done. Doing strike")
                    self.state_timer = 2000
                    self.state_index += 1
                    self.start_keystrike("motor1", down, up, 2, 100, 0)
                    #print(self.motor1C.strike_index)
                        
            elif self.state_index == 2:
                if self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    print(f"Tutor state changed to 2: finished\n")
                    self.poll_communication()
                    self.motor0C.strike_index = "done"
                    self.motor1C.strike_index = "done"
                    self.motor2C.strike_index = "done"
                    self.motor3C.strike_index = "done"
                    self.motor1C.motor.setPosition(up) 
                    self.motor2C.motor.setPosition(up) 
                    self.motor3C.motor.setPosition(up) 
                    self.motor1C.strike_target = up
                    self.motor2C.strike_target = up
                    self.motor3C.strike_target = up
                    
                    return -1 #animation finished
            #print(f"motor0: {self.motor0C.strike_index}")
            #print(f"motor1: {self.motor1C.strike_index}")

##########################################################
    def multislam(self):
        self.sendAnimation = "multislam"
        self.poll_communication()
        while self.step(EVENT_LOOP_DT) != -1:
            # Check the radio and/or network.
            self.poll_communication()
            for motor_key in ["motor1", "motor2", "motor3", "motor0"]: 
                #motor_key = self.motor_key
                motorC = self.motors[motor_key]
                # Read sensor value.
                motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()

            #motor0
            up = 0.0
            down = 0.8 
            # Update the activity state machine.    

            if self.state_index == 0:
                # if the timing is all derived from the low-level primitive, you may not need a timer here, but
                # it still might be a useful animation control
                if timer_expired or self.is_strike_done():
                    self.state_timer += 100
                    self.state_index += 1
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 500
                    self.state_index += 1
                    self.start_keystrike("motor3", 0.8, 0.4, 3, 100, 0)
                    self.start_keystrike("motor2", 0.8, 0.4, 3, 100, 0)
                    self.start_keystrike("motor1", 0.8, 0.4, 3, 100, 0)
            
            elif self.state_index == 2:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 50
                    self.state_index += 1
            
            elif self.state_index == 3:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 200
                    self.state_index += 1
                    self.start_keystrike("motor3", 0.8, 0.4, 3, 100, 0)
                    self.start_keystrike("motor2", 0.8, 0.4, 3, 100, 0)
                    self.start_keystrike("motor1", 0.8, 0.4, 3, 100, 0)
            
            elif self.state_index == 4:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 50
                    self.state_index += 1
            
            elif self.state_index == 5:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike("motor3", 0.8, 0.0, 3, 100, 500)
                    self.start_keystrike("motor2", 0.8, 0.0, 3, 100, 500)
                    self.start_keystrike("motor1", 0.8, 0.0, 3, 100, 500)
            
            elif self.state_index == 6:
                if timer_expired or self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    return -1 #animation finished
                
##########################################################
    def slam(self):
        self.sendAnimation = "slam"
        self.poll_communication()
        while self.step(EVENT_LOOP_DT) != -1:
            # Check the radio and/or network.
            self.poll_communication()
            for motor_key in ["motor1", "motor2", "motor3", "motor0"]: 
                #motor_key = self.motor_key
                motorC = self.motors[motor_key]
                # Read sensor value.
                motorC.pos = motorC.joint.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT
            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0
            # Update the motor controls
            self.update_keystrike()
            #Calibration Tables
            #motor3 placement (left finger from tutor POV)
            up = 0.0
            down = 0.8 
            # Update the activity state machine.    

            if self.state_index == 0:
                # if the timing is all derived from the low-level primitive, you may not need a timer here, but
                # it still might be a useful animation control
                if timer_expired or self.is_strike_done():
                    self.state_timer += 100
                    self.state_index += 1
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 500
                    self.state_index += 1
                    self.start_keystrike("motor3", 0.8, 0.4, 5, 100, 0)
                    self.start_keystrike("motor2", 0.8, 0.4, 5, 100, 0)
                    self.start_keystrike("motor1", 0.8, 0.4, 5, 100, 0)
            
            elif self.state_index == 2:
                if timer_expired or self.is_strike_done():
                    self.sendState = 2
                    self.poll_communication()
                    return -1 #animation finished


################################################################
# Start the script.
robot = Tutor()
robot.run()


