# Tutor2.py ver2

print("Tutor.py waking up.")

# Import the Webots simulator API.
from controller import Robot

# Import standard Python libraries.
import math, random, time

# Define the time step in milliseconds between controller updates.
EVENT_LOOP_DT = 10

################################################################
class Tutor(Robot):
    def __init__(self):

        super(Tutor, self).__init__()

        # Fetch handles for the sensor.
        self.j1  = self.getPositionSensor('sensor1')
        # Specify the sampling rate for the sensor.
        self.j1.enable(EVENT_LOOP_DT)
        # Fetch handle for the motor. 
        self.motor1 = self.getMotor('motor1')

        self.state_timer = 0        # timers in milliseconds
        self.state_index = 0        # current state
        self.pos = self.j1.getValue()
        
        self.strike_target = 0
        self.retreat_target = 0
        self.strike_speed = 0
        self.strike_index = 'down'
        self.pause_timer = 0
        

    #================================================================
    def start_keystrike(self, strike_position, retreat_position, speed, pause):
        self.strike_target = strike_position
        self.retreat_target = retreat_position
        self.strike_speed = speed
        self.strike_index = 'down'
        self.pause_timer = pause

        # This approach uses the low-level PID controller to generate each piecewise linear segment.
        # https://cyberbotics.com/doc/reference/motor?version=R2020b&tab-language=python#field-summary
        self.motor1.setVelocity(self.strike_speed)
        self.motor1.setPosition(self.strike_target)        

        return

    def update_keystrike(self):
        if self.strike_index == 'down':
            if abs(self.pos - self.strike_target) < 0.05:
                self.strike_index = 'pause'
        
        elif self.strike_index == 'pause':
            self.pause_timer -= EVENT_LOOP_DT
            self.motor1.setVelocity(0)
            if self.pause_timer <= 0:
                self.motor1.setPosition(self.retreat_target)
                self.motor1.setVelocity(self.strike_speed)
                self.strike_index = 'up'
        
        elif self.strike_index == 'up':
            if abs(self.pos - self.retreat_target) < 0.05:
                self.strike_index = 'done'

        # this structure could be extended if more phases were desired in each individual gesture

    def is_strike_done(self):
        """Returns true if an individual gesture is done."""
        return self.strike_index == 'done'
    
    #================================================================
    def run(self):

        while self.step(EVENT_LOOP_DT) != -1:
        
        # self.sim_time = self.getTime()
            # Read sensor value.
            self.pos = self.j1.getValue()
            # Update the activity state machine.
            self.state_timer -= EVENT_LOOP_DT

        # The behavioral state machine might be easier to maintain if the timer
        # code were directly integrated into each state:

            # Update the activity state machine timer.
            self.state_timer -= EVENT_LOOP_DT
            timer_expired = self.state_timer < 0

            # Update the motor controls
            self.update_keystrike()
            
            # Update the activity state machine.    
                   
            if self.state_index == 0:
                # if the timing is all derived from the low-level primitive, you may not need a timer here, but
                # it still might be a useful animation control
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(0.0, 0.4, .5, 500)
            
            elif self.state_index == 1:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 8000
                    self.state_index += 1
                    self.start_keystrike(0.0, 0.4, .5, 300)
            
            elif self.state_index == 2:
                if timer_expired or self.is_strike_done():
                    self.state_timer += 80000
                    self.state_index += 1
                    self.start_keystrike(0.0, 0.4, .5, 300)

            # ....
            print(self.state_index, self.strike_speed, self.strike_index, self.strike_target, self.retreat_target)
            
            # Also, please keep in mind that self.state_index values do not need to
            # be sequential or even numerical.


################################################################
# Start the script.
robot = Tutor()
robot.run()


